@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/retry.c,v 1.9 2006/03/09 15:10:16 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with retrying unsuccessful deliveries. */
@@ -12,6 +14,55 @@
 
 
 
+/*************************************************
+*         Check the ultimate address timeout     *
+*************************************************/
+
+/* This function tests whether a message has been on the queue longer than
+the maximum retry time for a particular host.
+
+Arguments:
+  host_key      the key to look up a host retry rule
+  domain        the domain to look up a domain retry rule
+  basic_errno   a specific error number, or zero if none
+  more_errno    additional data for the error
+  now           the time
+
+Returns:        TRUE if the ultimate timeout has been reached
+*/
+
+static BOOL
+ultimate_address_timeout(uschar *host_key, uschar *domain, int basic_errno,
+  int more_errno, time_t now)
+{
+BOOL address_timeout = TRUE;   /* no rule => timed out */
+
+retry_config *retry =
+  retry_find_config(host_key+2, domain, basic_errno, more_errno);
+
+if (retry != NULL && retry->rules != NULL)
+  {
+  retry_rule *last_rule;
+  for (last_rule = retry->rules;
+       last_rule->next != NULL;
+       last_rule = last_rule->next);
+  DEBUG(D_transport|D_retry)
+    debug_printf("now=%d received_time=%d diff=%d timeout=%d\n",
+      (int)now, received_time, (int)(now - received_time),
+      last_rule->timeout);
+  address_timeout = (now - received_time > last_rule->timeout);
+  }
+else
+  {
+  DEBUG(D_transport|D_retry)
+    debug_printf("no retry rule found: assume timed out\n");
+  }
+
+return address_timeout;
+}
+
+
+
 /*************************************************
 *     Set status of a host+address item          *
 *************************************************/
@@ -151,44 +202,25 @@ host. */
 
 if (host_retry_record != NULL)
   {
-  time_t now = time(NULL);
-
   *retry_host_key = host_key;
 
+  /* We have not reached the next try time. Check for the ultimate address
+  timeout if the host has not expired. */
+
   if (now < host_retry_record->next_try && !deliver_force)
     {
-    /* Check for ultimate address timeout if host not expired */
+    DEBUG(D_transport|D_retry)
+      debug_printf("host retry time not reached: checking ultimate address "
+        "timeout\n");
 
-    if (!host_retry_record->expired)
+    if (!host_retry_record->expired &&
+        ultimate_address_timeout(host_key, domain,
+          host_retry_record->basic_errno, host_retry_record->more_errno, now))
       {
-      BOOL address_timeout = FALSE;
-
-      retry_config *retry =
-        retry_find_config(host_key+2, domain,
-          host_retry_record->basic_errno,
-          host_retry_record->more_errno);
-
       DEBUG(D_transport|D_retry)
-        debug_printf("retry time not reached: checking ultimate address "
-          "timeout\n");
-
-      if (retry != NULL && retry->rules != NULL)
-        {
-        retry_rule *last_rule;
-        for (last_rule = retry->rules;
-             last_rule->next != NULL;
-             last_rule = last_rule->next);
-        if (now - received_time > last_rule->timeout) address_timeout = TRUE;
-        }
-      else address_timeout = TRUE;    /* No rule => timed out */
-
-      if (address_timeout)
-        {
-        DEBUG(D_transport|D_retry)
-          debug_printf("on queue longer than maximum retry for "
-            "address - allowing delivery\n");
-        return FALSE;
-        }
+        debug_printf("on queue longer than maximum retry for "
+          "address - allowing delivery\n");
+      return FALSE;
       }
 
     /* We have not hit the ultimate address timeout; host is unusable. */
@@ -206,15 +238,28 @@ if (host_retry_record != NULL)
   }
 
 /* It's OK to try the host. If there's a message-specific retry record, check
-for reaching its retry time (or forcing). If not, mark the host unusable. */
+for reaching its retry time (or forcing). If not, mark the host unusable,
+unless the ultimate address timeout has been reached. */
 
 if (message_retry_record != NULL)
   {
   *retry_message_key = message_key;
-  if (time(NULL) < message_retry_record->next_try && !deliver_force)
+  if (now < message_retry_record->next_try && !deliver_force)
     {
-    host->status = hstatus_unusable;
-    host->why = hwhy_retry;
+    DEBUG(D_transport|D_retry)
+      debug_printf("host+message retry time not reached: checking ultimate "
+        "address timeout\n");
+    if (!ultimate_address_timeout(host_key, domain, 0, 0, now))
+      {
+      host->status = hstatus_unusable;
+      host->why = hwhy_retry;
+      }
+    else
+      {
+      DEBUG(D_transport|D_retry)
+        debug_printf("on queue longer than maximum retry for "
+          "address - allowing delivery\n");
+      }
     return FALSE;
     }
   }
@@ -264,8 +309,16 @@ rti->message = addr->message;
 rti->flags = flags;
 
 DEBUG(D_transport|D_retry)
-  debug_printf("added retry item for %s: errno=%d %d flags=%d\n",
-    rti->key, rti->basic_errno, rti->more_errno, flags);
+  {
+  int letter = rti->more_errno & 255;
+  debug_printf("added retry item for %s: errno=%d more_errno=", rti->key,
+    rti->basic_errno);
+  if (letter == 'A' || letter == 'M')
+    debug_printf("%d,%c", (rti->more_errno >> 8) & 255, letter);
+  else
+    debug_printf("%d", rti->more_errno);
+  debug_printf(" flags=%d\n", flags);
+  }
 }
 
 
@@ -275,11 +328,13 @@ DEBUG(D_transport|D_retry)
 *************************************************/
 
 /* Search the in-store retry information for the first retry item that applies
-to a given destination. Remote Internet destinations are given as
-domain:IP-address. It doesn't seem necessary to go to that level of detail in
-retry rules, so we chop off the address and match on the domain part only.
-Local addresses are given as user@domain. The alternative destination is the
-domain name, which is always just a domain.
+to a given destination. If the key contains an @ we are probably handling a
+local delivery and have a complete address to search for; this happens when
+retry_use_local_part is set on a router. Otherwise, the key is likely to be a
+host name for a remote delivery, or a domain name for a local delivery. We
+prepend *@ on the front of it so that it will match a retry item whose address
+item pattern is independent of the local part. The alternate key, if set, is
+always just a domain, so we treat it likewise.
 
 Arguments:
   key          key for which retry info is wanted
@@ -294,32 +349,41 @@ retry_config *
 retry_find_config(uschar *key, uschar *alternate, int basic_errno,
   int more_errno)
 {
-int replace;
+int replace = 0;
 uschar *use_key, *use_alternate;
 uschar *colon = Ustrchr(key, ':');
 retry_config *yield;
 
-/* If there's a colon in the key, temporarily replace it with
-a zero to terminate the string there. */
+/* If there's a colon in the key, there are two possibilities:
+
+(1) This is a key for a host, ip address, and possibly port, in the format
+
+      hostname:ip+port
+
+    In this case, we temporarily replace the colon with a zero, to terminate
+    the string after the host name.
+
+(2) This is a key for a pipe, file, or autoreply delivery, in the format
+
+      pipe-or-file-or-auto:x@y
+
+    where x@y is the original address that provoked the delivery. The pipe or
+    file or auto will start with | or / or >, whereas a host name will start
+    with a letter or a digit. In this case we want to use the original address
+    to search for a retry rule. */
 
 if (colon != NULL)
   {
-  replace = ':';
-  }
-else
-  {
-  colon = key + Ustrlen(key);
-  replace = 0;
+  if (isalnum(*key))
+    replace = ':';
+  else
+    key = Ustrrchr(key, ':') + 1;   /* Take from the last colon */
   }
+
+if (replace == 0) colon = key + Ustrlen(key);
 *colon = 0;
 
-/* If the key contains an @ we are probably handling a local delivery and
-have a complete address to search for; this happens when retry_use_local_part
-is set on a router. Otherwise, the key is likely to be a host name for a
-remote delivery, or a domain name for a local delivery. We prepend *@ on the
-front of it so that it will match a retry item whose address item pattern is
-independent of the local part. The alternate key, if set, is always just a
-domain, so we treat it likewise. */
+/* Sort out the keys */
 
 use_key = (Ustrchr(key, '@') != NULL)? key : string_sprintf("*@%s", key);
 use_alternate = (alternate == NULL)? NULL : string_sprintf("*@%s", alternate);
@@ -329,16 +393,19 @@ use_alternate = (alternate == NULL)? NULL : string_sprintf("*@%s", alternate);
 for (yield = retries; yield != NULL; yield = yield->next)
   {
   uschar *plist = yield->pattern;
+  uschar *slist = yield->senders;
 
-  /* If a specific error is set for this item, check that we are
-  handling that specific error, and if so, check any additional
-  error information if required. Special code is required for quota
-  errors, as these can either be system quota errors, or Exim's own
-  quota imposition, which has a different error number. Full partitions
-  are also treated in the same way as quota errors. */
+  /* If a specific error is set for this item, check that we are handling that
+  specific error, and if so, check any additional error information if
+  required. */
 
   if (yield->basic_errno != 0)
     {
+    /* Special code is required for quota errors, as these can either be system
+    quota errors, or Exim's own quota imposition, which has a different error
+    number. Full partitions are also treated in the same way as quota errors.
+    */
+
     if (yield->basic_errno == ERRNO_EXIMQUOTA)
       {
       if ((basic_errno != ERRNO_EXIMQUOTA && basic_errno != errno_quota &&
@@ -347,7 +414,45 @@ for (yield = retries; yield != NULL; yield = yield->next)
         continue;
       }
 
+    /* The TLSREQUIRED error also covers TLSFAILURE. These are subtly different
+    errors, but not worth separating at this level. */
+
+    else if (yield->basic_errno == ERRNO_TLSREQUIRED)
+      {
+      if (basic_errno != ERRNO_TLSREQUIRED && basic_errno != ERRNO_TLSFAILURE)
+        continue;
+      }
+
+    /* Handle 4xx responses to MAIL, RCPT, or DATA. The code that was received
+    is in the 2nd least significant byte of more_errno (with 400 subtracted).
+    The required value is coded in the 2nd least significant byte of the
+    yield->more_errno field as follows:
+
+      255     => any 4xx code
+      >= 100  => the decade must match the value less 100
+      < 100   => the exact value must match
+    */
+
+    else if (yield->basic_errno == ERRNO_MAIL4XX ||
+             yield->basic_errno == ERRNO_RCPT4XX ||
+             yield->basic_errno == ERRNO_DATA4XX)
+      {
+      int wanted;
+      if (basic_errno != yield->basic_errno) continue;
+      wanted = (yield->more_errno >> 8) & 255;
+      if (wanted != 255)
+        {
+        int evalue = (more_errno >> 8) & 255;
+        if (wanted >= 100)
+          {
+          if ((evalue/10)*10 != wanted - 100) continue;
+          }
+        else if (evalue != wanted) continue;
+        }
+      }
+
     /* There are some special cases for timeouts */
+
     else if (yield->basic_errno == ETIMEDOUT)
       {
       if (basic_errno != ETIMEDOUT) continue;
@@ -367,22 +472,31 @@ for (yield = retries; yield != NULL; yield = yield->next)
       }
 
     /* Default checks for exact match */
+
     else
       {
-
       if (yield->basic_errno != basic_errno ||
          (yield->more_errno != 0 && yield->more_errno != more_errno))
        continue;
       }
     }
 
+  /* If the "senders" condition is set, check it. Note that sender_address may
+  be null during -brt checking, in which case we do not use this rule. */
+
+  if (slist != NULL && (sender_address == NULL ||
+      match_address_list(sender_address, TRUE, TRUE, &slist, NULL, -1, 0,
+        NULL) != OK))
+    continue;
+
   /* Check for a match between the address list item at the start of this retry
   rule and either the main or alternate keys. */
 
-  if (match_address_list(use_key, TRUE, &plist, NULL, -1, UCHAR_MAX+1) == OK ||
-       (use_alternate != NULL &&
-        match_address_list(use_alternate, TRUE, &plist, NULL, -1, UCHAR_MAX+1)
-          == OK))
+  if (match_address_list(use_key, TRUE, TRUE, &plist, NULL, -1, UCHAR_MAX+1,
+        NULL) == OK ||
+     (use_alternate != NULL &&
+      match_address_list(use_alternate, TRUE, TRUE, &plist, NULL, -1,
+        UCHAR_MAX+1, NULL) == OK))
     break;
   }
 
@@ -540,10 +654,12 @@ for (i = 0; i < 3; i++)
         DEBUG(D_retry)
           {
           if ((rti->flags & rf_host) != 0)
-            debug_printf("retry for %s (%s) = %s\n", rti->key,
-              addr->domain, retry->pattern);
+            debug_printf("retry for %s (%s) = %s %d %d\n", rti->key,
+              addr->domain, retry->pattern, retry->basic_errno,
+              retry->more_errno);
           else
-            debug_printf("retry for %s = %s\n", rti->key, retry->pattern);
+            debug_printf("retry for %s = %s %d %d\n", rti->key, retry->pattern,
+              retry->basic_errno, retry->more_errno);
           }
 
         /* Set up the message for the database retry record. Because DBM
@@ -579,6 +695,16 @@ for (i = 0; i < 3; i++)
         /* Compute how long this destination has been failing */
 
         failing_interval = now - retry_record->first_failed;
+        DEBUG(D_retry) debug_printf("failing_interval=%d message_age=%d\n",
+          failing_interval, message_age);
+
+        /* If the message has been on the queue longer than the recorded time
+        of failure, use the message's age instead. This can happen when some
+        messages can be delivered and others cannot; a successful delivery will
+        reset the first_failed time, and this can lead to a failing message
+        being retried too often. */
+
+        if (message_age > failing_interval) failing_interval = message_age;
 
         /* Search for the current retry rule. The cutoff time of the
         last rule is handled differently to the others. The rule continues
@@ -633,9 +759,16 @@ for (i = 0; i < 3; i++)
 
         This implements "timeout this rule if EITHER the host (or routing or
         directing) has been failing for more than the maximum time, OR if the
-        message has been on the queue for more than the maximum time." */
+        message has been on the queue for more than the maximum time."
 
-        if (received_time < retry_record->first_failed &&
+        February 2006: It is possible that this code is no longer needed
+        following the change to the retry calculation to use the message age if
+        it is larger than the time since first failure. It may be that the
+        expired flag is always set when the other conditions are met. However,
+        this is a small bit of code, and it does no harm to leave it in place,
+        just in case. */
+
+        if (received_time <= retry_record->first_failed &&
             addr == endaddr && !retry_record->expired && rule != NULL)
           {
           retry_rule *last_rule;
@@ -660,15 +793,25 @@ for (i = 0; i < 3; i++)
         if (rule == NULL) next_try = now; else
           {
           if (rule->rule == 'F') next_try = now + rule->p1;
-          else  /* assume rule = 'G' */
+          else  /* rule = 'G' or 'H' */
             {
             int last_predicted_gap =
               retry_record->next_try - retry_record->last_try;
             int last_actual_gap = now - retry_record->last_try;
             int lastgap = (last_predicted_gap < last_actual_gap)?
               last_predicted_gap : last_actual_gap;
-            next_try = now + ((lastgap < rule->p1)? rule->p1 :
-               (lastgap * rule->p2)/1000);
+            int next_gap = (lastgap * rule->p2)/1000;
+            if (rule->rule == 'G')
+              {
+              next_try = now + ((lastgap < rule->p1)? rule->p1 : next_gap);
+              }
+            else  /* The 'H' rule */
+              {
+              next_try = now + rule->p1;
+              if (next_gap > rule->p1)
+                next_try += random_number(next_gap - rule->p1)/2 +
+                  (next_gap - rule->p1)/2;
+              }
             }
           }
 
@@ -699,12 +842,18 @@ for (i = 0; i < 3; i++)
 
         DEBUG(D_retry)
           {
+          int letter = retry_record->more_errno & 255;
           debug_printf("Writing retry data for %s\n", rti->key);
           debug_printf("  first failed=%d last try=%d next try=%d expired=%d\n",
             (int)retry_record->first_failed, (int)retry_record->last_try,
             (int)retry_record->next_try, retry_record->expired);
-          debug_printf("  error %d %d: %s\n", retry_record->basic_errno,
-            retry_record->more_errno, retry_record->text);
+          debug_printf("  errno=%d more_errno=", retry_record->basic_errno);
+          if (letter == 'A' || letter == 'M')
+            debug_printf("%d,%c", (retry_record->more_errno >> 8) & 255,
+              letter);
+          else
+            debug_printf("%d", retry_record->more_errno);
+          debug_printf(" %s\n", retry_record->text);
           }
 
         (void)dbfn_write(dbm_file, rti->key, retry_record,
@@ -757,6 +906,9 @@ for (i = 0; i < 3; i++)
           setflag(addr, af_retry_timedout);
           addr->message = (addr->message == NULL)? US"retry timeout exceeded" :
             string_sprintf("%s: retry timeout exceeded", addr->message);
+          addr->user_message = (addr->user_message == NULL)?
+            US"retry timeout exceeded" :
+            string_sprintf("%s: retry timeout exceeded", addr->user_message);
           log_write(0, LOG_MAIN, "** %s%s%s%s: retry timeout exceeded",
             addr->address,
            (addr->parent == NULL)? US"" : US" <",
