@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/sieve.c,v 1.12 2005/06/17 10:47:05 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/sieve.c,v 1.20 2006/04/04 08:35:39 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Michael Haardt 2003-2005 */
+/* Copyright (c) Michael Haardt 2003-2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This code was contributed by Michael Haardt. */
@@ -28,10 +28,16 @@
 /* Undefine it for UNIX-style \n end-of-line terminators (default). */
 #undef RFC_EOL
 
-/* Define this for development of the subaddress Sieve extension.   */
+/* Define this for development of the Sieve extension "envelope-auth". */
+#undef ENVELOPE_AUTH
+
+/* Define this for development of the Sieve extension "notify".     */
+#undef NOTIFY
+
+/* Define this for the Sieve extension "subaddress".                */
 #define SUBADDRESS
 
-/* Define this for the vacation Sieve extension.                    */
+/* Define this for the Sieve extension "vacation".                  */
 #define VACATION
 
 /* Must be >= 1                                                     */
@@ -52,6 +58,13 @@ struct Sieve
   int keep;
   int require_envelope;
   int require_fileinto;
+#ifdef ENVELOPE_AUTH
+  int require_envelope_auth;
+#endif
+#ifdef NOTIFY
+  int require_notify;
+  struct Notification *notified;
+#endif
 #ifdef SUBADDRESS
   int require_subaddress;
 #endif
@@ -81,6 +94,14 @@ struct String
   int length;
   };
 
+struct Notification
+  {
+  struct String method;
+  struct String priority;
+  struct String message;
+  struct Notification *next;
+  };
+
 static int parse_test(struct Sieve *filter, int *cond, int exec);
 static int parse_commands(struct Sieve *filter, int exec, address_item **generated);
 
@@ -92,6 +113,8 @@ static uschar str_cc_c[]="Cc";
 static const struct String str_cc={ str_cc_c, 2 };
 static uschar str_bcc_c[]="Bcc";
 static const struct String str_bcc={ str_bcc_c, 3 };
+static uschar str_auth_c[]="auth";
+static const struct String str_auth={ str_auth_c, 4 };
 static uschar str_sender_c[]="Sender";
 static const struct String str_sender={ str_sender_c, 6 };
 static uschar str_resent_from_c[]="Resent-From";
@@ -102,6 +125,14 @@ static uschar str_fileinto_c[]="fileinto";
 static const struct String str_fileinto={ str_fileinto_c, 8 };
 static uschar str_envelope_c[]="envelope";
 static const struct String str_envelope={ str_envelope_c, 8 };
+#ifdef ENVELOPE_AUTH
+static uschar str_envelope_auth_c[]="envelope-auth";
+static const struct String str_envelope_auth={ str_envelope_auth_c, 13 };
+#endif
+#ifdef NOTIFY
+static uschar str_notify_c[]="notify";
+static const struct String str_notify={ str_notify_c, 6 };
+#endif
 #ifdef SUBADDRESS
 static uschar str_subaddress_c[]="subaddress";
 static const struct String str_subaddress={ str_subaddress_c, 10 };
@@ -139,6 +170,10 @@ static const struct String str_comparator_iascii_numeric={ str_comparator_iascii
 /*
 Arguments:
   src               UTF-8 string
+  dst               US-ASCII string
+
+Returns
+  dst
 */
 
 static struct String *quoted_printable_encode(const struct String *src, struct String *dst)
@@ -180,13 +215,8 @@ for (pass=0; pass<=1; ++pass)
       ||
         (
         (ch==9 || ch==32)
-#ifdef RFC_EOL
         && start+2<end
         && (*(start+1)!='\r' || *(start+2)!='\n')
-#else
-        && start+1<end
-        && *(start+1)!='\n'
-#endif
         )
       )
       {
@@ -196,29 +226,18 @@ for (pass=0; pass<=1; ++pass)
         *new++=*start;
       ++line;
       }
-#ifdef RFC_EOL
     else if (ch=='\r' && start+1<end && *(start+1)=='\n')
       {
       if (pass==0)
         {
         ++dst->length;
         line=0;
-        ++start;
         }
       else
         *new++='\n';
         line=0;
+      ++start;
       }
-#else
-    else if (ch=='\n')
-      {
-      if (pass==0)
-        ++dst->length;
-      else
-        *new++=*start;
-      ++line;
-      }
-#endif
     else
       {
       if (pass==0)
@@ -237,6 +256,173 @@ for (pass=0; pass<=1; ++pass)
 }
 
 
+/*************************************************
+*          Decode URI encoded string             *
+*************************************************/
+
+/*
+Arguments:
+  str               URI encoded string
+
+Returns
+  0                 Decoding successful
+ -1                 Encoding error
+*/
+
+static int uri_decode(struct String *str)
+{
+uschar *s,*t,*e;
+
+if (str->length==0) return 0;
+for (s=str->character,t=s,e=s+str->length; s<e; )
+  {
+  if (*s=='%')
+    {
+    if (s+2<e && isxdigit(*(s+1)) && isxdigit(*(s+2)))
+      {
+      *t++=((isdigit(*(s+1)) ? *(s+1)-'0' : tolower(*(s+1))-'a'+10)<<4)
+           | (isdigit(*(s+2)) ? *(s+2)-'0' : tolower(*(s+2))-'a'+10);
+      s+=3;
+      }
+    else return -1;
+    }
+  else
+    *t++=*s++;
+  }
+*t='\0';
+str->length=t-str->character;
+return 0;
+}
+
+
+/*************************************************
+*               Parse mailto URI                 *
+*************************************************/
+
+/*
+Parse mailto-URI.
+
+       mailtoURI   = "mailto:" [ to ] [ headers ]
+       to          = [ addr-spec *("%2C" addr-spec ) ]
+       headers     = "?" header *( "&" header )
+       header      = hname "=" hvalue
+       hname       = *urlc
+       hvalue      = *urlc
+
+Arguments:
+  filter      points to the Sieve filter including its state
+  uri         URI, excluding scheme
+
+Returns
+  1           URI is syntactically OK
+ -1           syntax error
+*/
+
+int parse_mailto_uri(struct Sieve *filter, const uschar *uri, string_item **recipient, struct String *body)
+{
+const uschar *start;
+struct String to,hname,hvalue;
+int capacity;
+string_item *new;
+
+if (*uri && *uri!='?')
+  for (;;)
+    {
+    /* match to */
+    for (start=uri; *uri && *uri!='?' && (*uri!='%' || *(uri+1)!='2' || tolower(*(uri+2))!='c'); ++uri);
+    if (uri>start)
+      {
+      capacity=0;
+      to.character=(uschar*)0;
+      to.length=0;
+      to.character=string_cat(to.character,&capacity,&to.length,start,uri-start);
+      to.character[to.length]='\0';
+      if (uri_decode(&to)==-1)
+        {
+        filter->errmsg=US"Invalid URI encoding";
+        return -1;
+        }
+        new=store_get(sizeof(string_item));
+        new->text=store_get(to.length+1);
+        if (to.length) memcpy(new->text,to.character,to.length);
+        new->text[to.length]='\0';
+        new->next=*recipient;
+        *recipient=new;
+      }
+    else
+      {
+      filter->errmsg=US"Missing addr-spec in URI";
+      return -1;
+      }
+    if (*uri=='%') uri+=3;
+    else break;
+    }
+if (*uri=='?')
+  {
+  ++uri;
+  for (;;)
+    {
+    /* match hname */
+    for (start=uri; *uri && (isalnum(*uri) || strchr("$-_.+!*'(),%",*uri)); ++uri);
+    if (uri>start)
+      {
+      capacity=0;
+      hname.character=(uschar*)0;
+      hname.length=0;
+      hname.character=string_cat(hname.character,&capacity,&hname.length,start,uri-start);
+      hname.character[hname.length]='\0';
+      if (uri_decode(&hname)==-1)
+        {
+        filter->errmsg=US"Invalid URI encoding";
+        return -1;
+        }
+      }
+    /* match = */
+    if (*uri=='=')
+      ++uri;
+    else
+      {
+      filter->errmsg=US"Missing equal after hname";
+      return -1;
+      }
+    /* match hvalue */
+    for (start=uri; *uri && (isalnum(*uri) || strchr("$-_.+!*'(),%",*uri)); ++uri);
+    if (uri>start)
+      {
+      capacity=0;
+      hvalue.character=(uschar*)0;
+      hvalue.length=0;
+      hvalue.character=string_cat(hvalue.character,&capacity,&hvalue.length,start,uri-start);
+      hvalue.character[hvalue.length]='\0';
+      if (uri_decode(&hvalue)==-1)
+        {
+        filter->errmsg=US"Invalid URI encoding";
+        return -1;
+        }
+      }
+    if (hname.length==2 && strcmp(CS hname.character,"to")==0)
+      {
+      new=store_get(sizeof(string_item));
+      new->text=store_get(hvalue.length+1);
+      if (hvalue.length) memcpy(new->text,hvalue.character,hvalue.length);
+      new->text[hvalue.length]='\0';
+      new->next=*recipient;
+      *recipient=new;
+      }
+    else if (hname.length==4 && strcmp(CS hname.character,"body")==0)
+      *body=hvalue;
+    if (*uri=='&') ++uri;
+    else break;
+    }
+  }
+if (*uri)
+  {
+  filter->errmsg=US"Syntactically invalid URI";
+  return -1;
+  }
+return 1;
+}
+
 /*************************************************
 *          Octet-wise string comparison          *
 *************************************************/
@@ -311,8 +497,7 @@ while (nl>0 && hl>0)
   if (hc&0x80) return 0;
 #endif
   /* tolower depends on the locale and only ASCII case must be insensitive */
-  if ((nc&0x80) || (hc&0x80)) { if (nc!=hc) return 0; }
-  else if ((nc>='A' && nc<='Z' ? nc|0x20 : nc) != (hc>='A' && hc<='Z' ? hc|0x20 : hc)) return 0;
+  if ((nc>='A' && nc<='Z' ? nc|0x20 : nc) != (hc>='A' && hc<='Z' ? hc|0x20 : hc)) return 0;
   ++n;
   ++h;
   --nl;
@@ -323,7 +508,7 @@ return (match_prefix ? nl==0 : nl==0 && hl==0);
 
 
 /*************************************************
-*        Octet-wise glob pattern search          *
+*              Glob pattern search               *
 *************************************************/
 
 /*
@@ -333,231 +518,104 @@ Arguments:
 
 Returns:      0               needle not found in haystack
               1               needle found
+              -1              pattern error
 */
 
-static int eq_octetglob(const struct String *needle,
-  const struct String *haystack)
+static int eq_glob(const struct String *needle,
+  const struct String *haystack, int ascii_caseless, int match_octet)
 {
-struct String n,h;
+const uschar *n,*h,*nend,*hend;
+int may_advance=0;
 
-n=*needle;
-h=*haystack;
-while (n.length)
+n=needle->character;
+h=haystack->character;
+nend=n+needle->length;
+hend=h+haystack->length;
+while (n<nend)
   {
-  switch (n.character[0])
+  if (*n=='*')
     {
-    case '*':
-      {
-      int currentLength;
-
-      ++n.character;
-      --n.length;
-      /* The greedy match is not yet well tested.  Some day we may */
-      /* need to refer to the matched parts, so the code is already */
-      /* prepared for that. */
-#if 1
-      /* greedy match */
-      currentLength=h.length;
-      h.character+=h.length;
-      h.length=0;
-      while (h.length<=currentLength)
-        {
-        if (eq_octetglob(&n,&h)) return 1;
-        else /* go back one octet */
-          {
-          --h.character;
-          ++h.length;
-          }
-        }
-      return 0;
-#else
-      /* minimal match */
-      while (h.length)
-        {
-        if (eq_octetglob(&n,&h)) return 1;
-        else /* advance one octet */
-          {
-          ++h.character;
-          --h.length;
-          }
-        }
-      break;
-#endif
-      }
-    case '?':
-      {
-      if (h.length)
-        {
-        ++h.character;
-        --h.length;
-        ++n.character;
-        --n.length;
-        }
-      else return 0;
-      break;
-      }
-    case '\\':
-      {
-      ++n.character;
-      --n.length;
-      /* FALLTHROUGH */
-      }
-    default:
-      {
-      if
-        (
-        h.length==0 ||
-#if !HAVE_ICONV
-        (h.character[0]&0x80) || (n.character[0]&0x80) ||
-#endif
-        h.character[0]!=n.character[0]
-        ) return 0;
-      else
-        {
-        ++h.character;
-        --h.length;
-        ++n.character;
-        --n.length;
-        };
-      }
+    ++n;
+    may_advance=1;
     }
-  }
-return (h.length==0);
-}
-
-
-/*************************************************
-*   ASCII case-insensitive glob pattern search   *
-*************************************************/
-
-/*
-Arguments:
-  needle      UTF-8 pattern to search ...
-  haystack    ... inside the haystack
-
-Returns:      0               needle not found in haystack
-              1               needle found
-*/
-
-static int eq_asciicaseglob(const struct String *needle,
-  const struct String *haystack)
-{
-struct String n,h;
-
-n=*needle;
-h=*haystack;
-while (n.length)
-  {
-  switch (n.character[0])
+  else
     {
-    case '*':
-      {
-      int currentLength;
+    const uschar *npart,*hpart;
 
-      ++n.character;
-      --n.length;
-      /* The greedy match is not yet well tested.  Some day we may */
-      /* need to refer to the matched parts, so the code is already */
-      /* prepared for that. */
-#if 1
-      /* greedy match */
-      currentLength=h.length;
-      h.character+=h.length;
-      h.length=0;
-      while (h.length<=currentLength)
+    /* Try to match a non-star part of the needle at the current */
+    /* position in the haystack.                                 */
+    match_part:
+    npart=n;
+    hpart=h;
+    while (npart<nend && *npart!='*') switch (*npart)
+      {
+      case '?':
         {
-        if (eq_asciicaseglob(&n,&h)) return 1;
-        else /* go back one UTF-8 character */
+        if (hpart==hend) return 0;
+        if (match_octet)
+          ++hpart;
+        else
           {
-          if (h.length==currentLength) return 0;
-          --h.character;
-          ++h.length;
-          if (h.character[0]&0x80)
+          /* Match one UTF8 encoded character */
+          if ((*hpart&0xc0)==0xc0)
             {
-            while (h.length<currentLength && (*(h.character-1)&0x80))
-              {
-              --h.character;
-              ++h.length;
-              }
-            }
-          }
-        }
-      /* NOTREACHED */
-#else
-      while (h.length)
-        {
-        if (eq_asciicaseglob(&n,&h)) return 1;
-        else /* advance one UTF-8 character */
-          {
-          if (h.character[0]&0x80)
-            {
-            while (h.length && (h.character[0]&0x80))
-              {
-              ++h.character;
-              --h.length;
-              }
+            ++hpart;
+            while (hpart<hend && ((*hpart&0xc0)==0x80)) ++hpart;
             }
           else
-            {
-            ++h.character;
-            --h.length;
-            }
+            ++hpart;
           }
+        ++npart;
+        break;
         }
-      break;
-#endif
-      }
-    case '?':
-      {
-      if (h.length)
+      case '\\':
         {
-        ++n.character;
-        --n.length;
-        /* advance one UTF-8 character */
-        if (h.character[0]&0x80)
+        ++npart;
+        if (npart==nend) return -1;
+        /* FALLTHROUGH */
+        }
+      default:
+        {
+        if (hpart==hend) return 0;
+        /* tolower depends on the locale, but we need ASCII */
+        if
+          (
+#if !HAVE_ICONV
+          (*hpart&0x80) || (*npart&0x80) ||
+#endif
+          ascii_caseless
+          ? ((*npart>='A' && *npart<='Z' ? *npart|0x20 : *npart) != (*hpart>='A' && *hpart<='Z' ? *hpart|0x20 : *hpart))
+          : *hpart!=*npart
+          )
           {
-          while (h.length && (h.character[0]&0x80))
+          if (may_advance)
+            /* string match after a star failed, advance and try again */
             {
-            ++h.character;
-            --h.length;
+            ++h;
+            goto match_part;
             }
+          else return 0;
           }
         else
           {
-          ++h.character;
-          --h.length;
-          }
+          ++npart;
+          ++hpart;
+          };
         }
-      else return 0;
-      break;
       }
-    case '\\':
+    /* at this point, a part was matched successfully */
+    if (may_advance && npart==nend && hpart<hend)
+      /* needle ends, but haystack does not: if there was a star before, advance and try again */
       {
-      ++n.character;
-      --n.length;
-      /* FALLTHROUGH */
-      }
-    default:
-      {
-      char nc,hc;
-
-      if (h.length==0) return 0;
-      nc=n.character[0];
-      hc=h.character[0];
-#if !HAVE_ICONV
-      if ((hc&0x80) || (nc&0x80)) return 0;
-#endif
-      /* tolower depends on the locale and only ASCII case must be insensitive */
-      if ((nc&0x80) || (hc&0x80)) { if (nc!=hc) return 0; }
-      else if ((nc>='A' && nc<='Z' ? nc|0x20 : nc) != (hc>='A' && hc<='Z' ? hc|0x20 : hc)) return 0;
-      ++h.character;
-      --h.length;
-      ++n.character;
-      --n.length;
+      ++h;
+      goto match_part;
       }
+    h=hpart;
+    n=npart;
+    may_advance=0;
     }
   }
-return (h.length==0);
+return (h==hend ? 1 : may_advance);
 }
 
 
@@ -620,6 +678,7 @@ switch (relop)
 
 /*
 Arguments:
+  filter      points to the Sieve filter including its state
   needle      UTF-8 pattern or string to search ...
   haystack    ... inside the haystack
   co          comparator to use
@@ -715,12 +774,20 @@ switch (mt)
       {
       case COMP_OCTET:
         {
-        if (eq_octetglob(needle,haystack)) r=1;
+        if ((r=eq_glob(needle,haystack,0,1))==-1)
+          {
+          filter->errmsg=CUS "syntactically invalid pattern";
+          return -1;
+          }
         break;
         }
       case COMP_EN_ASCII_CASEMAP:
         {
-        if (eq_asciicaseglob(needle,haystack)) r=1;
+        if ((r=eq_glob(needle,haystack,1,1))==-1)
+          {
+          filter->errmsg=CUS "syntactically invalid pattern";
+          return -1;
+          }
         break;
         }
       default:
@@ -885,6 +952,10 @@ new_addr->next = *generated;
 *************************************************/
 
 /*
+Unfold the header field as described in RFC 2822 and remove all
+leading and trailing white space, then perform MIME decoding and
+translate the header field to UTF-8.
+
 Arguments:
   value       returned value of the field
   header      name of the header field
@@ -902,20 +973,17 @@ value->length=0;
 value->character=(uschar*)0;
 
 t=r=s=expand_string(string_sprintf("$rheader_%s",quote(header)));
-while (*r==' ') ++r;
+while (*r==' ' || *r=='\t') ++r;
 while (*r)
   {
   if (*r=='\n')
-    {
     ++r;
-    while (*r==' ' || *r=='\t') ++r;
-    if (*r) *t++=' ';
-    }
   else
     *t++=*r++;
   }
-*t++='\0';
-value->character=rfc2047_decode(s,TRUE,US"utf-8",'\0',&value->length,&errmsg);
+while (t>s && (*(t-1)==' ' || *(t-1)=='\t')) --t;
+*t='\0';
+value->character=rfc2047_decode(s,check_rfc2047_length,US"utf-8",'\0',&value->length,&errmsg);
 }
 
 
@@ -1085,6 +1153,7 @@ if (*filter->pc=='"') /* quoted string */
       int foo=data->length;
 
       ++filter->pc;
+      /* that way, there will be at least one character allocated */
       data->character=string_cat(data->character,&dataCapacity,&foo,CUS "",1);
       return 1;
       }
@@ -1096,6 +1165,15 @@ if (*filter->pc=='"') /* quoted string */
       }
     else /* regular character */
       {
+#ifdef RFC_EOL
+      if (*filter->pc=='\r' && *(filter->pc+1)=='\n') ++filter->line;
+#else
+      if (*filter->pc=='\n')
+        {
+        data->character=string_cat(data->character,&dataCapacity,&data->length,US"\r",1);
+        ++filter->line;
+        }
+#endif
       data->character=string_cat(data->character,&dataCapacity,&data->length,filter->pc,1);
       filter->pc++;
       }
@@ -1151,7 +1229,10 @@ else if (Ustrncmp(filter->pc,CUS "text:",5)==0) /* multiline string */
       if (*filter->pc=='.' && *(filter->pc+1)=='\n') /* end of string */
 #endif
         {
-        data->character=string_cat(data->character,&dataCapacity,&data->length,CUS "",1);
+        int foo=data->length;
+
+        /* that way, there will be at least one character allocated */
+        data->character=string_cat(data->character,&dataCapacity,&foo,CUS "",1);
 #ifdef RFC_EOL
         filter->pc+=3;
 #else
@@ -1949,6 +2030,9 @@ else if (parse_identifier(filter,CUS "envelope"))
                   <envelope-part: string-list> <key-list: string-list>
 
   envelope-part is case insensitive "from" or "to"
+#ifdef ENVELOPE_AUTH
+  envelope-part =/ "auth"
+#endif
   */
 
   enum Comparator comparator=COMP_EN_ASCII_CASEMAP;
@@ -2011,7 +2095,7 @@ else if (parse_identifier(filter,CUS "envelope"))
     return -1;
     }
   *cond=0;
-  for (e=env; e->character; ++e)
+  for (e=env; e->length!=-1 && !*cond; ++e)
     {
     const uschar *envelopeExpr=CUS 0;
     uschar *envelope=US 0;
@@ -2044,6 +2128,23 @@ else if (parse_identifier(filter,CUS "envelope"))
         case ADDRPART_DOMAIN: envelopeExpr=CUS "$domain"; break;
         }
       }
+#ifdef ENVELOPE_AUTH
+    else if (eq_asciicase(e,&str_auth,0))
+      {
+      switch (addressPart)
+        {
+        case ADDRPART_ALL: envelopeExpr=CUS "$authenticated_sender"; break;
+#ifdef SUBADDRESS
+        case ADDRPART_USER:
+#endif
+        case ADDRPART_LOCALPART: envelopeExpr=CUS "${local_part:$authenticated_sender}"; break;
+        case ADDRPART_DOMAIN: envelopeExpr=CUS "${domain:$authenticated_sender}"; break;
+#ifdef SUBADDRESS
+        case ADDRPART_DETAIL: envelopeExpr=CUS 0; break;
+#endif
+        }
+      }
+#endif
     else
       {
       filter->errmsg=CUS "invalid envelope string";
@@ -2179,6 +2280,11 @@ while (*filter->pc)
       filter->errmsg=CUS "missing test";
       return -1;
       }
+    if ((filter_test != FTEST_NONE && debug_selector != 0) ||
+        (debug_selector & D_filter) != 0)
+      {
+      if (exec) debug_printf("if %s\n",cond?"true":"false");
+      }
     m=parse_block(filter,exec ? cond : 0, generated);
     if (m==-1 || m==2) return m;
     if (m==0)
@@ -2200,6 +2306,11 @@ while (*filter->pc)
           filter->errmsg=CUS "missing test";
           return -1;
           }
+        if ((filter_test != FTEST_NONE && debug_selector != 0) ||
+            (debug_selector & D_filter) != 0)
+          {
+          if (exec) debug_printf("elsif %s\n",cond?"true":"false");
+          }
         m=parse_block(filter,exec && unsuccessful ? cond : 0, generated);
         if (m==-1 || m==2) return m;
         if (m==0)
@@ -2312,7 +2423,7 @@ while (*filter->pc)
     fileinto-command =  "fileinto" { fileinto-options } string ";"
     fileinto-options =
     fileinto-options =) [ ":copy" ]
-   */
+    */
 
     struct String folder;
     uschar *s;
@@ -2366,6 +2477,146 @@ while (*filter->pc)
       }
     if (parse_semicolon(filter)==-1) return -1;
     }
+#ifdef NOTIFY
+  else if (parse_identifier(filter,CUS "notify"))
+    {
+    /*
+    notify-command =  "notify" { notify-options } ";"
+    notify-options =  [":method" string]
+                      [":priority" string]
+                      [":message" string]
+    */
+
+    int m;
+    struct String method;
+    struct String priority;
+    struct String message;
+    struct Notification *already;
+    string_item *recipient;
+    struct String body;
+    uschar *envelope_from,*envelope_to;
+
+    if (!filter->require_notify)
+      {
+      filter->errmsg=CUS "missing previous require \"notify\";";
+      return -1;
+      }
+    method.character=(uschar*)0;
+    method.length=-1;
+    priority.character=(uschar*)0;
+    priority.length=-1;
+    message.character=(uschar*)0;
+    message.length=-1;
+    recipient=NULL;
+    body.length=-1;
+    body.character=(uschar*)0;
+    envelope_from=expand_string("$sender_address");
+    envelope_to=expand_string("$local_part_prefix$local_part$local_part_suffix@$domain");
+    for (;;)
+      {
+      if (parse_white(filter)==-1) return -1;
+      if (parse_identifier(filter,CUS ":method")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&method))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "method string expected";
+          return -1;
+          }
+        if (method.length>7 && strncmp(method.character,"mailto:",7)==0)
+          {
+          if (parse_mailto_uri(filter,method.character+7,&recipient,&body)==-1) return -1;
+          }
+        }
+      else if (parse_identifier(filter,CUS ":priority")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&priority))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "priority string expected";
+          return -1;
+          }
+        }
+      else if (parse_identifier(filter,CUS ":message")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&message))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "message string expected";
+          return -1;
+          }
+        }
+      else break;
+      }
+    if (parse_semicolon(filter)==-1) return -1;
+
+    if (method.length==-1)
+      {
+        if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
+          {
+          debug_printf("Ignoring method-less notification.\n");
+          }
+      }
+    else
+      {
+      for (already=filter->notified; already; already=already->next)
+        {
+        if (already->method.length==method.length
+            && (method.length==-1 || strcmp(already->method.character,method.character)==0)
+            && already->priority.length==priority.length
+            && (priority.length==-1 || strcmp(already->priority.character,priority.character)==0)
+            && already->message.length==message.length
+            && (message.length==-1 || strcmp(already->message.character,message.character)==0))
+          break;
+        }
+      if (already==(struct Notification*)0)
+        /* New notification, process it */
+        {
+        struct Notification *sent;
+        sent=store_get(sizeof(struct Notification));
+        sent->method=method;
+        sent->priority=priority;
+        sent->message=message;
+        sent->next=filter->notified;
+        filter->notified=sent;
+        if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
+          {
+          debug_printf("Notification to `%s'.\n",method.character);
+          }
+        if (exec)
+          {
+          string_item *p;
+          header_line *h;
+          int pid,fd;
+
+          if ((pid = child_open_exim2(&fd,envelope_to,envelope_to))>=1)
+            {
+            FILE *f;
+
+            f = fdopen(fd, "wb");
+            fprintf(f,"From: %s\n",envelope_to);
+            for (p=recipient; p; p=p->next) fprintf(f,"To: %s\n",p->text);
+            for (h = header_list; h != NULL; h = h->next)
+              if (h->type == htype_received) fprintf(f,"%s",h->text);
+            fprintf(f,"Subject: %s\n",message.length==-1 ? CUS "notification" : message.character);
+            fprintf(f,"\n");
+            if (body.length>0) fprintf(f,"%s\n",body.character);
+            fflush(f);
+            (void)fclose(f);
+            (void)child_close(pid, 0);
+            }
+          }
+        }
+      else
+        {
+        if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
+          {
+          debug_printf("Repeated notification to `%s' ignored.\n",method.character);
+          }
+        }
+      }
+    }
+#endif
 #ifdef VACATION
   else if (parse_identifier(filter,CUS "vacation"))
     {
@@ -2552,11 +2803,11 @@ while (*filter->pc)
         md5_start(&base);
         md5_end(&base, key.character, key.length, digest);
         for (i = 0; i < 16; i++) sprintf(CS (hexdigest+2*i), "%02X", digest[i]);
-        if (filter_test != FTEST_NONE)
+        if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
           {
           debug_printf("Sieve: mail was personal, vacation file basename: %s\n", hexdigest);
           }
-        else
+        if (filter_test == FTEST_NONE)
           {
           capacity=Ustrlen(filter->vacation_directory);
           start=capacity;
@@ -2568,11 +2819,22 @@ while (*filter->pc)
 
           if (subject.length==-1)
             {
-            expand_header(&subject,&str_subject);
-            capacity=6;
-            start=6;
-            subject.character=string_cat(US"Auto: ",&capacity,&start,subject.character,subject.length);
-            subject.length=start;
+            uschar *subject_def;
+
+            subject_def=expand_string(US"${if def:header_subject {true}{false}}");
+            if (Ustrcmp(subject_def,"true")==0)
+              {
+              expand_header(&subject,&str_subject);
+              capacity=6;
+              start=6;
+              subject.character=string_cat(US"Auto: ",&capacity,&start,subject.character,subject.length);
+              subject.length=start;
+              }
+            else
+              {
+              subject.character=US"Automated reply";
+              subject.length=Ustrlen(subject.character);
+              }
             }
 
           /* add address to list of generated addresses */
@@ -2590,9 +2852,9 @@ while (*filter->pc)
           else
             addr->reply->from = from.character;
           /* Allocation is larger than neccessary, but enough even for split MIME words */
-          buffer_capacity=16+4*subject.length;
+          buffer_capacity=32+4*subject.length;
           buffer=store_get(buffer_capacity);
-          addr->reply->subject=parse_quote_2047(subject.character, subject.length, US"utf-8", buffer, buffer_capacity);
+          addr->reply->subject=parse_quote_2047(subject.character, subject.length, US"utf-8", buffer, buffer_capacity, TRUE);
           addr->reply->oncelog=once;
           addr->reply->once_repeat=days*86400;
 
@@ -2601,16 +2863,12 @@ while (*filter->pc)
           if (reason_is_mime)
             {
             uschar *mime_body,*reason_end;
-#ifdef RFC_EOL
             static const uschar nlnl[]="\r\n\r\n";
-#else
-            static const uschar nlnl[]="\n\n";
-#endif
 
             for
               (
               mime_body=reason.character,reason_end=reason.character+reason.length;
-              mime_body<(reason_end-sizeof(nlnl)-1) && memcmp(mime_body,nlnl,sizeof(nlnl)-1);
+              mime_body<(reason_end-(sizeof(nlnl)-1)) && memcmp(mime_body,nlnl,(sizeof(nlnl)-1));
               ++mime_body
               );
             capacity = 0;
@@ -2619,7 +2877,7 @@ while (*filter->pc)
             addr->reply->headers[start] = '\0';
             capacity = 0;
             start = 0;
-            if (mime_body+(sizeof(nlnl)-1)<reason_end) mime_body+=sizeof(nlnl)-1;
+            if (mime_body+(sizeof(nlnl)-1)<reason_end) mime_body+=(sizeof(nlnl)-1);
             else mime_body=reason_end-1;
             addr->reply->text = string_cat(NULL,&capacity,&start,mime_body,reason_end-mime_body);
             addr->reply->text[start] = '\0';
@@ -2638,7 +2896,7 @@ while (*filter->pc)
             }
           }
         }
-        else if (filter_test != FTEST_NONE)
+        else if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
           {
           debug_printf("Sieve: mail was not personal, vacation would ignore it\n");
           }
@@ -2673,6 +2931,13 @@ filter->line=1;
 filter->keep=1;
 filter->require_envelope=0;
 filter->require_fileinto=0;
+#ifdef ENVELOPE_AUTH
+filter->require_envelope_auth=0;
+#endif
+#ifdef NOTIFY
+filter->require_notify=0;
+filter->notified=(struct Notification*)0;
+#endif
 #ifdef SUBADDRESS
 filter->require_subaddress=0;
 #endif
@@ -2738,6 +3003,12 @@ while (parse_identifier(filter,CUS "require"))
     {
     if (eq_octet(check,&str_envelope,0)) filter->require_envelope=1;
     else if (eq_octet(check,&str_fileinto,0)) filter->require_fileinto=1;
+#ifdef ENVELOPE_AUTH
+    else if (eq_octet(check,&str_envelope_auth,0)) filter->require_envelope_auth=1;
+#endif
+#ifdef NOTIFY
+    else if (eq_octet(check,&str_notify,0)) filter->require_notify=1;
+#endif
 #ifdef SUBADDRESS
     else if (eq_octet(check,&str_subaddress,0)) filter->require_subaddress=1;
 #endif
@@ -2838,12 +3109,12 @@ if (parse_start(&sieve,1,generated)==1)
   if (sieve.keep)
     {
     add_addr(generated,US"inbox",1,0,0,0);
-    msg = string_sprintf("Keep");
+    msg = string_sprintf("Implicit keep");
     r = FF_DELIVERED;
     }
-    else
+  else
     {
-    msg = string_sprintf("No keep");
+    msg = string_sprintf("No implicit keep");
     r = FF_DELIVERED;
     }
   }
