@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/transports/appendfile.h,v 1.4 2006/02/07 11:19:03 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -19,15 +21,20 @@ typedef struct {
   uschar *quota_filecount;
   uschar *quota_size_regex;
   uschar *quota_warn_threshold;
+  uschar *mailbox_size_string;
+  uschar *mailbox_filecount_string;
+  uschar *maildir_dir_regex;
   uschar *maildir_tag;
   uschar *mailstore_prefix;
   uschar *mailstore_suffix;
   uschar *check_string;
   uschar *escape_string;
   uschar *file_format;
-  int   quota_value;
+  off_t quota_value;
+  off_t quota_warn_threshold_value;
+  off_t mailbox_size_value;
+  int   mailbox_filecount_value;
   int   quota_filecount_value;
-  int   quota_warn_threshold_value;
   int   mode;
   int   dirmode;
   int   lockfile_mode;
@@ -83,4 +90,8 @@ extern appendfile_transport_options_block appendfile_transport_option_defaults;
 extern BOOL appendfile_transport_entry(transport_instance *, address_item *);
 extern void appendfile_transport_init(transport_instance *);
 
+/* Function that is shared with tf_maildir.c */
+
+extern off_t  check_dir_size(uschar *, int *, const pcre *);
+
 /* End of transports/appendfile.h */
