@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/transports/tf_maildir.c,v 1.8 2006/02/07 11:19:03 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2012 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions in support of the use of maildirsize files for handling quotas in
@@ -27,49 +29,46 @@ calculations are not hard wired in, but are supplied as a regex. */
 *************************************************/
 
 /* This function is called at the start of a maildir delivery, to ensure that
-all the relevant directories exist. It also creates a maildirfolder file if the
-base directory matches a given pattern.
+all the relevant directories exist.
 
 Argument:
   path              the base directory name
   addr              the address item (for setting an error message)
   create_directory  true if we are allowed to create missing directories
   dirmode           the mode for created directories
-  maildirfolder_create_regex
-                    the pattern to match for maildirfolder creation
 
 Returns:            TRUE on success; FALSE on failure
 */
 
 BOOL maildir_ensure_directories(uschar *path, address_item *addr,
-  BOOL create_directory, int dirmode, uschar *maildirfolder_create_regex)
+  BOOL create_directory, int dirmode)
 {
 int i;
 struct stat statbuf;
-const char *subdirs[] = { "/tmp", "/new", "/cur" };
+char *subdirs[] = { "/tmp", "/new", "/cur" };
 
 DEBUG(D_transport)
   debug_printf("ensuring maildir directories exist in %s\n", path);
 
 /* First ensure that the path we have is a directory; if it does not exist,
 create it. Then make sure the tmp, new & cur subdirs of the maildir are
-there. If not, fail. This aborts the delivery (even though the cur subdir is
+there. If not, fail which aborts the delivery (even though the cur subdir is
 not actually needed for delivery). Handle all 4 directory tests/creates in a
 loop so that code can be shared. */
 
 for (i = 0; i < 4; i++)
   {
   int j;
-  const uschar *dir, *mdir;
+  uschar *dir, *mdir;
 
   if (i == 0)
     {
-    mdir = CUS"";
+    mdir = US"";
     dir = path;
     }
   else
     {
-    mdir = CUS subdirs[i-1];
+    mdir = US subdirs[i-1];
     dir = mdir + 1;
     }
 
@@ -136,55 +135,7 @@ for (i = 0; i < 4; i++)
     }
   }
 
-/* If the basic path matches maildirfolder_create_regex, we are dealing with
-a subfolder, and should ensure that a maildirfolder file exists. */
-
-if (maildirfolder_create_regex != NULL)
-  {
-  const uschar *error;
-  int offset;
-  const pcre *regex;
-
-  DEBUG(D_transport) debug_printf("checking for maildirfolder requirement\n");
-
-  regex = pcre_compile(CS maildirfolder_create_regex, PCRE_COPT,
-    (const char **)&error, &offset, NULL);
-
-  if (regex == NULL)
-    {
-    addr->message = string_sprintf("appendfile: regular expression "
-      "error: %s at offset %d while compiling %s", error, offset,
-      maildirfolder_create_regex);
-    return FALSE;
-    }
-
-  if (pcre_exec(regex, NULL, CS path, Ustrlen(path), 0, 0, NULL, 0) >= 0)
-    {
-    uschar *fname = string_sprintf("%s/maildirfolder", path);
-    if (Ustat(fname, &statbuf) == 0)
-      {
-      DEBUG(D_transport) debug_printf("maildirfolder already exists\n");
-      }
-    else
-      {
-      int fd = Uopen(fname, O_WRONLY|O_APPEND|O_CREAT, 0600);
-      if (fd < 0)
-        {
-        addr->message = string_sprintf("appendfile: failed to create "
-          "maildirfolder file in %s directory: %s", path, strerror(errno));
-        return FALSE;
-        }
-      (void)close(fd);
-      DEBUG(D_transport) debug_printf("created maildirfolder file\n");
-      }
-    }
-  else
-    {
-    DEBUG(D_transport) debug_printf("maildirfolder file not required\n");
-    }
-  }
-
-return TRUE;   /* Everything exists that should exist */
+return TRUE;   /* All directories exist */
 }
 
 
@@ -212,7 +163,7 @@ uschar buffer[256];
 sprintf(CS buffer, "%d 1\n", size);
 len = Ustrlen(buffer);
 (void)lseek(fd, 0, SEEK_END);
-len = write(fd, buffer, len);
+(void)write(fd, buffer, len);
 DEBUG(D_transport)
   debug_printf("added '%.*s' to maildirsize file\n", len-1, buffer);
 }
@@ -390,7 +341,7 @@ the same thing. */
 filename = string_sprintf("%s/maildirsize", path);
 
 DEBUG(D_transport) debug_printf("looking for maildirsize in %s\n", path);
-fd = Uopen(filename, O_RDWR|O_APPEND, ob->mode ? ob->mode : 0600);
+fd = Uopen(filename, O_RDWR|O_APPEND, 0);
 if (fd < 0)
   {
   if (errno != ENOENT) return -1;
@@ -554,10 +505,10 @@ else
     FALSE);
 
   (void)gettimeofday(&tv, NULL);
-  tempname = string_sprintf("%s/tmp/" TIME_T_FMT ".H%luP%lu.%s",
-    path, tv.tv_sec, tv.tv_usec, (long unsigned) getpid(), primary_hostname);
+  tempname = string_sprintf("%s/tmp/%lu.H%luP%lu.%s", path, tv.tv_sec,
+    tv.tv_usec, getpid(), primary_hostname);
 
-  fd = Uopen(tempname, O_RDWR|O_CREAT|O_EXCL, ob->mode ? ob->mode : 0600);
+  fd = Uopen(tempname, O_RDWR|O_CREAT|O_EXCL, 0600);
   if (fd >= 0)
     {
     (void)sprintf(CS buffer, OFF_T_FMT "S,%dC\n" OFF_T_FMT " %d\n",
@@ -582,7 +533,7 @@ else
       "a later subdirectory modification\n");
     (void)Uunlink(filename);
     (void)close(fd);
-    fd = -2;
+    fd = -1;
     }
   }
 
